<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class supports OSB listing products
 *
 * @package    OSB
 * @subpackage Product
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2010 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 *
 * Column Definitions:
 * + price_type: 0=One Time, 1=Recurring, 2=Trial
 */
class Model_Product extends ORM_OSB {
	// @todo this doesnt have our site_id when getting the translation
	protected $_has_many = array(
		'product_translate'=>array('far_key'=>'id'),
		'service'=>array('far_key'=>'id'),
		'invoice'=>array('through'=>'invoice_item'),
	);

	protected $_sorting = array(
		'position'=>'asc',
	);

	protected $_display_filters = array(
		'price_base'=>array(
			array('Tax::add',array(':value')),
			array('Currency::display',array(':value')),
		),
		'price_type'=>array(
			array('StaticList_PriceType::display',array(':value')),
		),
		'status'=>array(
			array('StaticList_YesNo::display',array(':value')),
		),
		'taxable'=>array(
			array('StaticList_YesNo::display',array(':value')),
		),
	);

	// Our attributes that are arrays, we'll convert/unconvert them
	protected $_serialize_column = array(
		'price_group',
	);

	/**
	 * Return the object of the product plugin
	 */
	public function plugin() {
		if (! $this->prod_plugin_file)
			return NULL;

		if (! is_numeric($this->prod_plugin_data))
			throw new Kohana_Exception('Missing plugin_id for :product (:type)',array(':product'=>$this->id,':type'=>$this->prod_plugin_file));

		return ORM::factory(sprintf('Product_Plugin_%s',$this->prod_plugin_file),$this->prod_plugin_data);
	}

	/**
	 * Get the product name, after translating
	 * @todo This needs to be improved to find the right language item.
	 */
	public function name() {
		return $this->product_translate->find()->display('name');
	}

	/**
	 * This will render the product feature summary information
	 */
	public function feature_summary() {
		return (is_null($plugin = $this->plugin())) ? HTML::nbsp('') : $plugin->feature_summary();
	}

	/**
	 * Return the best price to the uesr based on the users group's memberships
	 * @todo This needs to be tested with more than one price group enabled
	 */
	public function get_price_array() {
		if (! $this->loaded())
			throw new Kohana_Exception('Call to :method where no object loaded?',array(':method'=>__METHOD__));

		// Figure out our eligable groups
		// @todo Need to work out our default groups elsewhere, not in product
		// All users are members of the all user group "0"
		$groups = array(0);
		if (Auth::instance()->logged_in())
			foreach (Auth::instance()->get_user()->group->find_all() as $go)
				array_push($groups,$go->id);

		// Work out the best price for the user
		$price = array();
		if (is_array($this->price_group))
		foreach ($this->price_group as $bill_freq => $pg) {
			if (isset($pg['show']) AND $pg['show'])
				foreach ($groups as $gid) {
					if (! empty($pg[$gid])) {
						if (empty($price[$bill_freq]['price_base'])
							OR ($pg[$gid]['price_base'] AND $price[$bill_freq]['price_base'] > $pg[$gid]['price_base'])) {
							$price[$bill_freq]['price_setup'] = $pg[$gid]['price_setup'];
							$price[$bill_freq]['price_base'] = $pg[$gid]['price_base'];
						}
					}
				}
		}

		// @todo Ugly hack
		return $price ? $price : array('0'=>array('price_base'=>0,'price_setup'=>0));
	}

	/**
	 * Test if the product is a TRIAL product
	 * (price_type == 2)
	 *
	 * @return boolean
	 */
	public function is_trial() {
		if ($this->price_type == 2)
			return TRUE;
		else
			return FALSE;
	}

	public function show_thumb() {
		$mediapath = Route::get('default/media');

		$thumbfile = sprintf('prod_thmb_%s',$this->id);

		// @todo This needs to be optimised. By nice if find_files could return the HTML path too.
		if (Kohana::find_file('media/img/thumbnails',$thumbfile,'png')) {
			$thumb = $mediapath->uri(array('file'=>'img/thumbnails/'.$thumbfile.'.png'));

			// @todo Change the ALT to the product name.
			echo HTML::image($thumb,array('alt'=>_('Thumb Nail')));
		} else
			echo '';
	}

	/**
	 * Enable the plugin to store data
	 */
	public function admin_update() {
		if (is_null($plugin = $this->plugin()))
			return NULL;
		else
			return $plugin->admin_update();
	}

	/**
	 * Is price shown for a specific period
	 */
	public function isPriceShown($p) {
		$x = $this->keyget('price_group',$p);

		return (isset($x['show']) AND $x['show']) ? TRUE : FALSE;
	}

	/**
	 * Return the configured price groups for this product
	 */
	public function availPriceGroups() {
		// @todo This needs to be worked out dynamically
		return array(0,2);
	}

	/**
	 * Return the available pricing options
	 */
	public function availPriceOptions() {
		// @todo This needs to be worked out dynamically
		return array('price_base','price_setup');
	}

	/**
	 * List the number of services using this product
	 */
	public function count_services() {
		return $this->service->list_count(TRUE);
	}

	/**
	 * List the number of invoices using this product
	 */
	public function count_invoices() {
		return $this->invoice->list_count(TRUE);
	}

	/**
	 * Return the price for the particle group and price option for the period
	 */
	public function price($grp,$period,$option) {
		$x = $this->keyget('price_group',$period);

		return isset($x[$grp][$option]) ? $x[$grp][$option] : NULL;
	}

	/**
	 * Return the products for a given category
	 * @todo This shouldnt be here.
	 */
	public function list_category($cat,$active=TRUE) {
		$results = array();

		$cats = $active ? $this->_where_active() : $this;

		foreach ($cats->find_all() as $po) {
			if ($c = unserialize($po->avail_category) AND in_array($cat,$c))
				array_push($results,$po);
		}

		Sort::MAsort($results,'position,price_base');
		return $results;
	}
}
?>
